from pyopenscad import *
from polysurface import spline_double_divide, Offset, SurfaceFromCurves

spline_me = True

h = -50
d = -30
bline = [(-90, d+110, 25+h), (-75, d+140, 35+h), (-60, d+180, 30+h), (-45, d+220, 20+h), (-20, d+260, -5+h)]

tbj1 = (-0, d+160, 45+h)
tbj2 = (-35, d+160, 40+h)
tbj3 = bline[1] # (-70, d+140, 42+h)
tbj4 = bline[0]

def test_bonnet3(thick, scl):

    p0 = [(-0, d+110, 55+h), (-0, d+140, 50+h), tbj1]
    p1 = [(-35, d+110, 55+h), (-35, d+140, 40+h), tbj2]
    p2 = [(-75, d+105, 50+h), (-65, d+120, 35+h), tbj3]
    p3 = [bline[0], bline[0], tbj4] 

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def test_bonnet2(thick, scl):

    p0 = [tbj1, (-0, d+170, 42+h), (-0, d+180, 40+h), (-0, d+220, 30+h), (-0, d+260, -5+h)]
    p1 = [tbj2, (-35, d+170, 35+h), (-35, d+180, 30+h), (-35, d+220, 20+h), bline[4]]
    p2 = [tbj3, bline[1], bline[2], bline[3], bline[4]]
    p3 = bline

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

tb12 = (-105, d+115, 15+h)
tb13 = (-124, d+120, 15+h)
blo = [tb13, (-125, d+140, 20+h), (-125, d+180, 30+h), (-125, d+220, 40+h), (-115, d+250, 35+h)]
tbf1 = (-70, d+253, -0+h)

def test_bonnet1(thick, scl):

    #p0 = [(0, d+110, 55+h), (0, d+140, 50+h), (0, d+180, 40+h), (0, d+220, 30+h), (0, d+260, -5+h)]
    p1 = bline
    p2 = [tb12, (-90, d+140, 20+h), (-80, d+180, 18+h), (-75, d+220, 10+h), tbf1]
    p3 = blo

    plist = [p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def nose(thick, scl):

    p0 = [rsbc1[0], (0, d+260, -5+h)]
    p1 = [rsbc1[1],(0, d+268, -78)]
    p2 = [rsbc1[2],(0, 230, -100)]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

tspc = [
    (-96, 220, -60,),
    (-100, 230, -58,),
    (-104, 233, -58,),
        (-126, 233, -58,),
    (-130, 230, -58,),
    (-132, d+250, -10+h),
    ]

tspc1 = (-122, d+250, 30+h)

tsb1 = (-130, d+120, -10+h)
tsb2 = (-132, d+133, -7+h)

def test_side_bonnet_behind_wheel(thick, scl):

    p0 = [blo[0], (-124,d+105, -5+h), (-124,d+100, -90), (-124,d+105, -125)]
    p1 = [tsb1, (-130, d+114, -20+h), (-130, d+114, -90), (-130, d+120, -125)]
    p2 = [tsb2, (-132, d+125, -30+h), (-132, d+127, -100), (-132, d+132, -125)]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def test_side_bonnet_top_panel(thick, scl):

    p0 = blo
    p1 = [tsb1, (-130, d+150, 15+h), (-130, d+180, 25+h), (-130, d+220, 30+h), tspc1]
    p2 = [tsb2, (-132, d+155, 10+h), (-132, d+190, 20+h), (-132, d+225, 15+h), tspc[5]]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

fw0 = (-94, 163, -29,)
fw1 = (-100, 163, -20,)
fw2 = (-104, 163, -16,)
fw3 = (-126, 163, -16,)
fw4 = (-130, 163, -20,)
fw5 = (-136, 163, -29,)

def test_front_wing_front_inner(thick, scl):

    p0 = [
(-96, 220, -128,),
(-96, 225, -93,),
tspc[0], 
    ]
    p1 = [
(-100, 228, -128,),
(-100, 232, -93,),
tspc[1], 
    ]
    p2 = [
(-104, 233, -128,),
(-104, 237, -93,),
tspc[2], 
    ]
    p3 = [
    (-126, 233, -128,),
    (-126, 237, -93,),
tspc[3], 
        ]
    p4 = [
(-130, 228, -128,),
(-130, 232, -93,),
tspc[4], 
    ]
    p5 = [
(-134, 220, -128,),
(-134, 225, -93,),
tspc[5], 
    ]

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, thick, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

tspcr = [
    (-102, -238, -60,),
    (-104, -240, -58,),
    (-106, -243, -58,),
        (-131, -243, -58,),
    (-135, -240, -58,),
    (-137, -238, -58),
    ]

def test_rear_wing_rear_inner(thick, scl):

    p0 = [
(-102, -242, -125,),
(-102, -245, -100,),
(-102, -245, -80,),
tspcr[0], 
    ]
    p1 = [
(-104, -245, -125,),
(-104, -248, -100,),
(-104, -248, -80,),
tspcr[1], 
    ]
    p2 = [
(-106, -248, -125,),
(-106, -251, -100,),
(-106, -251, -80,),
tspcr[2], 
    ]
    p3 = [
    (-131, -248, -125,),
    (-131, -251, -100,),
    (-131, -251, -80,),
tspcr[3], 
        ]
    p4 = [
(-135, -245, -125,),
(-135, -248, -100,),
(-135, -248, -80,),
tspcr[4], 
    ]
    p5 = [
(-137, -242, -125,),
(-137, -245, -100,),
(-137, -245, -80,),
tspcr[5], 
    ]

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, thick, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_wing_over_scoop(thick, scl):

    p0 = [(-110, -251, -125), (-105, -246, -125), (-100, -235, -125)]
    p1 = [(-80, -251, -125), (-80, -246, -125), (-80, -235, -125)]
    p2 = [(-60, -251, -100), (-60, -246, -100), (-60, -235, -100)]
    p3 = [(-40, -251, -100), (-40, -246, -100), (-40, -235, -100)]
    p4 = [(0, -251, -100), (0, -246, -100), (0, -235, -100)]
    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_wing_under_scoop(thick, scl):

    p0 = [(-110, -251, -125), (-105, -246, -125), (-100, -235, -125)]
    p1 = [(0, -251, -125), (0, -246, -125), (0, -235, -125)]
    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_under_scoop4(thick, scl):

    p0 = [(-110, 233, -100), (-105, 228, -100), (-100, 220, -100)]
    p1 = [(0, 237, -100), (0, 228, -100), (0, 220, -100)]
    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_under_scoop6(thick, scl):

    p0 = [(-110, 233, -100), (-105, 228, -100), (-100, 220, -100)]
    p1 = [(-80, 237, -100), (-80, 228, -100), (-80, 220, -100)]
    p2 = [(-60, 237, -60), (-60, 228, -60), (-60, 220, -60)]
    p3 = [(-40, 237, -60), (-40, 228, -60), (-40, 220, -60)]
    p4 = [(0, 237, -60), (0, 228, -60), (0, 220, -60)]
    plist = [p0, p1, p2, p3, p4]
    #p0 = [(-110, 233, -60), (-105, 228, -60), (-100, 220, -60)]
    #p1 = [(0, 237, -60), (0, 228, -60), (0, 220, -60)]
    #plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_under_scoop5(thick, scl):

    p0 = [(-110, 233, -100), (-105, 228, -100), (-100, 220, -100)]
    p1 = [(-80, 237, -100), (-80, 228, -100), (-80, 220, -100)]
    p2 = [(-60, 237, -78), (-60, 228, -78), (-60, 220, -78)]
    p3 = [(-40, 237, -78), (-40, 228, -78), (-40, 220, -78)]
    p4 = [(0, 237, -78), (0, 228, -78), (0, 220, -78)]
    plist = [p0, p1, p2, p3, p4]
    #p0 = [(-110, 233, -60), (-105, 228, -60), (-100, 220, -60)]
    #p1 = [(0, 237, -60), (0, 228, -60), (0, 220, -60)]
    #plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_under_scoop3(thick, scl):

    p0 = [(-110, 233, -78), (-105, 228, -78), (-100, 220, -78)]
    p1 = [(-80, 237, -78), (-80, 228, -78), (-80, 220, -78)]
    p2 = [(-60, 237, -60), (-60, 228, -60), (-60, 220, -60)]
    p3 = [(-40, 237, -60), (-40, 228, -60), (-40, 220, -60)]
    p4 = [(0, 237, -60), (0, 228, -60), (0, 220, -60)]
    plist = [p0, p1, p2, p3, p4]
    #p0 = [(-110, 233, -60), (-105, 228, -60), (-100, 220, -60)]
    #p1 = [(0, 237, -60), (0, 228, -60), (0, 220, -60)]
    #plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_under_scoop2(thick, scl):

    p0 = [(-110, 233, -78), (-105, 228, -78), (-100, 220, -78)]
    p1 = [(0, 237, -78), (0, 228, -78), (0, 220, -78)]
    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_over_scoop(thick, scl):

    p0 = [(-110, 233, -128), (-105, 228, -128), (-100, 220, -128)]
    p1 = [(-80, 237, -128), (-80, 228, -128), (-80, 220, -128)]
    p2 = [(-60, 237, -100), (-60, 228, -100), (-60, 220, -100)]
    p3 = [(-40, 237, -100), (-40, 228, -100), (-40, 220, -100)]
    p4 = [(0, 237, -100), (0, 228, -100), (0, 220, -100)]
    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_under_scoop(thick, scl):

    p0 = [(-110, 233, -128), (-105, 228, -128), (-100, 220, -128)]
    p1 = [(0, 237, -128), (0, 228, -128), (0, 220, -128)]
    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

hdp = [blo[4], (-100, 225, -35,), tspc[0]]

def headlight_panel(thick, scl):

    p0 = [
        hdp[0],
        (-115, 227, -20,),
        (-120, 230, -30,),
            (-120, 230, -30,),
        (-115, 227, -20,),
        blo[4],
        ]

    p2 = [
        hdp[1],
        (-104, 230, -30,),
        (-110, 235, -40,),
            (-125, 235, -40,),
        (-128, 230, -40,),
        tspc1,
        ]

    p3 = tspc

    plist = [p0, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_side_bonnet_panel(thick, scl):

    p0 = [
        hdp[0],
        hdp[0],
        hdp[0],
        hdp[0],
        ]

    p2 = [
        hdp[1],
        (-80, 230, -50,),
        (-70, 230, -52,),
        tbf1,
        ]

    p3 = [
        hdp[2],
        (-90, 225, -55,),
        (-75, 228, -55,),
        bline[4]
        ]

    plist = [p0, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

tsrpc = [
    (-108, 220-345, -70,),
    (-112, 230-345, -68,),
    (-116, 233-345, -68,),
        (-126, 233-340, -68,),
    (-132, 230-340, -68,),
    (-136, -115, -70),
    ]

def test_rear_wing_front_inner(thick, scl):

    p0 = [
(-98, 220-340, -128,),
(-98, 225-360, -93,),
tsrpc[0], 
    ]
    p1 = [
(-102, 228-340, -128,),
(-102, 232-360, -93,),
tsrpc[1], 
    ]
    p2 = [
(-106, 233-340, -128,),
(-106, 237-360, -93,),
tsrpc[2], 
    ]
    p3 = [
    (-128, 233-340, -128,),
    (-128, 237-340, -93,),
tsrpc[3], 
        ]
    p4 = [
(-132, 228-340, -128,),
(-132, 232-340, -93,),
tsrpc[4], 
    ]
    p5 = [
(-136, 220-340, -128,),
(-136, 225-340, -93,),
tsrpc[5], 
    ]

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, thick, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def test_panel(thick, scl):

    h = -50
    d = -30
    p0 = [(0, d+110, 55+h), (0, d+140, 50+h), (0, d+180, 40+h), (0, d+220, 30+h), (0, d+260, -5+h)]
    p1 = [(35, d+110, 55+h), (35, d+140, 40+h), (35, d+180, 30+h), (35, d+220, 20+h), (35, d+260, -5+h)]
    p2 = [(75, d+105, 50+h), (90, d+140, 20+h), (75, d+180, 20+h), (75, d+220, 10+h), (75, d+260, -5+h)]
    p3 = [(125, d+100, -0+h), (125, d+140, 20+h), (125, d+180, 30+h), (125, d+220, 40+h), (125, d+260, 40+h)]

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)


def hubcap_front_left(scl, mirror):
    r = rotate([90, 0, 90])
    r.add(cylinder(h=27*scl, r=45*scl, center=True))
    t = translate(v=[-119*scl*mirror, 163*scl, -97*scl])
    t.add(r)
    return t

def hubcap_back_left(scl, mirror):
    r = rotate([90, 0, 90])
    r.add(cylinder(h=26*scl, r=45*scl, center=True))
    t = translate(v=[-120*scl*mirror, -185*scl, -97*scl])
    t.add(r)
    return t

def bullbars_front(scl, mirror):
    u = union()
    r = rotate([90, 0, 90])
    for bb in range(3):
        r.add(cylinder(h=90*scl, r=3*scl, center=True))
        t = translate(v=[-45*scl*mirror, 230*scl, -(97-bb*19)*scl])
        t.add(r)
        u.add(t)

    r = rotate([0, 0, 90])
    r.add(cylinder(h=40*scl, r=3*scl, center=True))
    t = translate(v=[-88*scl*mirror, 230*scl, -(97-19)*scl])
    t.add(r)
    u.add(t)

    r = rotate([0, 0, 90])
    r.add(cylinder(h=40*scl, r=3*scl, center=True))
    t = translate(v=[-55*scl*mirror, 230*scl, -(97-19)*scl])
    t.add(r)
    u.add(t)
    return u

def bullbars_rear(scl, mirror):
    u = union()
    r = rotate([90, 0, 90])
    for bb in range(3):
        r.add(cylinder(h=105*scl, r=3*scl, center=True))
        t = translate(v=[-55*scl*mirror, (-243+(bb-1)*3)*scl, -(97-bb*19)*scl])
        t.add(r)
        u.add(t)

    r = rotate([-10, 0, 0])
    r.add(cylinder(h=40*scl, r=3*scl, center=True))
    t = translate(v=[-88*scl*mirror, -246*scl, -(97-19)*scl])
    t.add(r)
    u.add(t)

    r = rotate([-10, 0, 0])
    r.add(cylinder(h=40*scl, r=3*scl, center=True))
    t = translate(v=[-55*scl*mirror, -246*scl, -(97-19)*scl])
    t.add(r)
    u.add(t)
    return u

def wheel_front_left(scl, mirror):
    r = rotate([90, 0, 90])
    r.add(cylinder(h=26*scl, r=60*scl, center=True))
    t = translate(v=[-119*scl*mirror, 163*scl, -97*scl])
    t.add(r)
    return t

def wheel_back_left(scl, mirror):
    r = rotate([90, 0, 90])
    r.add(cylinder(h=25*scl, r=60*scl, center=True))
    t = translate(v=[-120*scl*mirror, -185*scl, -97*scl])
    t.add(r)
    return t

rarl0 = (-132, -115, -10)
rarl1 = (-120, -125, -10)
rarl2 = (-120, -145, 0)
rarl3 = (-120, -170, 6)
rarl4 = (-115, -200, 12)
rarl5 = (-100, -220, 16)
rarl6 = (-83, -250, 25)
rarl7 = (-82, -260, 25)

rarr0 = rarl0
rarr1 = (-136, -115, -10)
rarr2 = (-134, -145, 0)
rarr3 = (-132, -170, 6)
rarr4 = (-130, -200, 12)
rarr5 = (-120, -235, 20)
rarr6 = (-100, -250, 20)
rarr7 = rarl7

rarf1 = [ (-122, -115, -10),  (-122, -115, -30), tsrpc[0]]
rarf2 = [ rarl0,  (-132, -115, -30), tsrpc[5]]
rarb = [ rarl7, (-82, -255, -35), (-125, -243, -64) ]

rarel = [rarl0, rarl1, rarl2, rarl3, rarl4, rarl5, rarl6, rarl7]
rarei = [rarr0, rarr1, rarr2, rarr3, rarr4, rarr5, rarr6, rarr7]

rarb0 = (-103, -115, 6)
rarb1 = (-100, -125, 6)
rarb2 = (-90, -145, 8)
rarb3 = (-115, -150, 3)
rarb4 = (-100, -180, 12)
rarb5 = (-86, -205, 26)
rarb6 = rarb5 #(-75, -240, 20)
rarb7 = rarb6 # (-75, -260, 25)
rarb8 = rarb7

rareb = [rarb0, rarb1, rarb2, rarb3, rarb4, rarb5, rarb6, rarb7, rarb8]

def rear_arch_front(thick, scl):

    p0 = tsrpc
    p1 = [rarf1[1], (-124, -112, -30), (-126, -110, -30), (-128, -110, -30), (-130, -112, -30), rarf2[1]]
    p2 = [rarf1[0], (-124, -112, -10), (-126, -110, -10), (-128, -110, -10), (-132, -112, -10), rarf2[0]]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_spoiler(thick, scl):

    p0 = [rarb5, rarl6, rarl7]
    p1 = [(-65, -230, 32), (-65, -245, 30), (-65, -262, 26)]
    p2 = [(-45, -240, 32), (-45, -245, 30), (-45, -263, 26)]
    p3 = [(-25, -248, 26), (-25, -252, 26), (-25, -263, 26)]
    p4 = [(0, -248, 26), (0, -252, 26), (0, -263, 26)]

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_arch_body_joiner(thick, scl):

    p0 = rarel + [rarl7]
    p1 = rareb

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_arch_lid(thick, scl):

    p0 = rarel
    p1 = rarei

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rai1 = [ rarl1,  (-115, -125, -30), (-110, -135, -50)]

def rear_arch_inside(thick, scl):

    p0 = rarf1
    p1 = rai1
    p2 = [ rarl2,  (-110, -145, -20), (-110, -145, -42)]
    p3 = [ rarl3, (-110, -170, 0), (-110, -170, -28) ]
    p4 = [ rarl4, (-110, -200, 0), (-110, -200, -28) ]
    p5 = [ rarl5, (-100, -220, -20), (-110, -235, -40) ]
    p6 = [ rarl6, (-82, -239, -40), (-105, -238, -55) ]
    p7 = rarb

    plist = [p0, p1, p2, p3, p4, p5, p6, p7]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

ras1 = [ rarr1,  (-136, -115, -30), (-136, -115, -70)]

def rear_arch_side(thick, scl):

    p0 = rarf2
    p1 = ras1
    p2 = [ rarr2,  (-134, -145, -20), (-136, -145, -42)]
    p3 = [ rarr3, (-132, -170, -17), (-136, -170, -28) ]
    p4 = [ rarr4, (-130, -200, -17), (-136, -200, -28) ]
    p5 = [ rarr5, (-115, -235, -30), (-134, -225, -40) ]
    p6 = [ rarr6, (-90, -245, -30), (-128, -238, -55) ]
    p7 = rarb

    plist = [p0, p1, p2, p3, p4, p5, p6, p7]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_grille_1(thick, scl, trans, offset):

    p0 = [ (-82, -258, 10), (-0, -258, 10) ]
    p1 = [ (-88, -250, 10+offset), (-0, -250, 10+offset) ]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl,
                                trans=trans)

def rear_grille(t, thick, scl):

    for i in range(5):
        tr = (0, 1.0*i, -11*i)
        offset = -(i*.1+1)
        t.add(rear_grille_1(thick, scl, tr, offset))
    return t

def rear_grille_2(thick, scl, trans, offset):

    p0 = [ (-114, -243, -48), (-20, -243, -48) ]
    p1 = [ (-105, -237, -48+offset/2), (-20, -237, -48+offset/2) ]
    p2 = [ (-102, -230, -48+offset), (-20, -230, -48+offset) ]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl,
                                trans=trans)

def rear_grille_upright(thick, scl, trans):

    p0 = [ (-35, -238, -49), (-35, -226, -52) ]
    p1 = [ (-35, -252, -96), (-35, -240, -99) ]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl, trans=trans)

def rear_lower_grille(t, thick, scl):

    for i in range(4):
        tr = (0, -2.4*i, -12*i)
        if i < 2:
            tr = (0, tr[1] + 4-2.0*i, tr[2])
        offset = -(i*.1+3)
        t.add(rear_grille_2(thick, scl, tr, offset))
    for i in range(1,2):
        tr = (-20.0*i, 0, 0)
        t.add(rear_grille_upright(thick, scl, tr))
    return t

def front_grille_2(thick, scl, trans, offset, width, woff):

    p0 = [ (-90, 236, -85), (width, 236, -85) ]
    p1 = [ (-90-woff/2, 229, -85+offset/2), (width+woff/2, 229, -85+offset/2) ]
    p2 = [ (-90-woff, 222, -85+offset), (width+woff, 222, -85+offset) ]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl,
                                trans=trans)

def front_grille_upright(thick, scl, trans, offset):

    p0 = [ (-31, 235, -85), (-31+offset, 222, -79) ]
    p1 = [ (-31, 247, -122), (-31+offset, 234, -119) ]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl, trans=trans)

def front_lower_grille(t, thick, scl):

    for i in range(4):
        tr = (0, 3*i, -12*i)
        #if i < 2:
        #    tr = (0, tr[1] + 4-2.0*i, tr[2])
        #if i > 2:
        #    width = 5
        #else:
        #    width = -30.0
        width = -30.0
        offset = (4-i*0.5)
        t.add(front_grille_2(thick, scl, tr, offset, width, -5.0))
    for i in range(4):
        tr = (-20.0*i, 0, 0)
        if i == 0:
            offset = -5.0
        elif i == 3:
            offset = 5.0
        else:
            offset = 0
        t.add(front_grille_upright(thick, scl, tr, offset))
    return t

def front_grille_1(thick, scl, trans, offset):

    p0 = [ (-118, 245, -40), (-0, 245, -40) ]
    p1 = [ (-118, 240, -40+offset), (-0, 240, -40+offset) ]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl,
                                trans=trans)

def front_grille(t, thick, scl):

    for i in range(4):
        tr = (0, -3.45*i, -9.0*i)
        offset = (i*.4+1)
        t.add(front_grille_1(thick, scl, tr, offset))
    return t

fw0 = (-94, 163, -29,)
fw1 = (-100, 163, -20,)
fw2 = (-104, 163, -16,)
fw3 = (-126, 163, -16,)
fw4 = (-130, 163, -20,)
fw5 = (-136, 163, -29,)

fwbb0 = (-96, 98, -123,)
fwbbl = [
fwbb0,
(-96, 103, -93,),
(-96, 108, -67,),
(-96, 123, -42,),
(-94, 145, -29,),
fw0,
    ]

fwab0 = (-134, 108, -123,)

fwbbr = [
fwab0,
(-134, 103, -93,),
(-134, 108, -67,),
(-134, 123, -42,),
(-136, 145, -29,),
fw5,
    ]

fwba1 = (-105, 60, -123,)
fwba2 = (-120, 35, -123,)
fwba3 = (-125, 35, -123,)
fwba4 = (-130, 70, -123,)


fwb1 = (-100, 96, -123,)
fwb2 = (-104, 95, -123,)
fwb3 = (-126, 95, -123,)
fwb4 = (-130, 100, -123,)

fwbl = [ fwbb0, fwbb0]

def front_wing_floor_strut(thick, scl):

    p0 = [ fwbb0, fwba1 ]
    p1 = [ (-85, 95, -123), (-70, 60, -123)]
    p2 = [ (-50, 130, -123), (-50, 80, -123)]

    #plist = [p0, p1, p2, p3, p4, p5]
    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_aero_base(thick, scl):

    p0 = fwbl
    p1 = [ fwba1, fwb1,]
    p2 = [ fwba2, fwb2, ]
    p3 = [ fwba3, fwb3, ]
    p4 = [ fwba4, fwb4, ]
    p5 = [ fwab0,fwab0,]

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_back_aero_outer(thick, scl):

    p3 = fwbij
    p4 = [
fwba4,
(-130, 86, -93,),
(-130, 98, -63,),
(-130, 117, -36,),
(-130, 145, -20,),
fw4,
    ]
    p5 = fwbbr

    plist = [p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

fwbij = [
fwba3,
(-126, 70, -93,),
(-126, 90, -63,),
(-126, 115, -32,),
(-126, 130, -16,),
fw3,
    ]
def front_wing_back_aero_inner(thick, scl):

    p0 = fwbbl
    p1 = [
fwba1,
(-105, 70, -93,),
(-102, 90, -63,),
(-100, 110, -36,),
(-100, 130, -20,),
fw1,
    ]
    p2 = [
fwba2,
(-105, 70, -93,),
(-104, 90, -63,),
(-104, 110, -32,),
(-104, 130, -16,),
fw2,
    ]
    p3 = fwbij

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

fwbj = [
fwb3,
(-126, 91, -93,),
(-126, 95, -63,),
(-126, 115, -32,),
(-126, 145, -16,),
fw3,
    ]
def front_wing_back_inner(thick, scl):

    p0 = fwbbl
    p1 = [
fwb1, 
(-100, 96, -93,),
(-100, 98, -63,),
(-100, 117, -36,),
(-100, 145, -20,),
fw1,
    ]
    p2 = [
fwb2,
(-104, 91, -93,),
(-104, 95, -63,),
(-104, 115, -32,),
(-104, 145, -16,),
fw2,
    ]
    p3 = fwbj

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_back_outer(thick, scl):

    p3 = fwbj
    p4 = [
fwb4,
(-130, 96, -93,),
(-130, 98, -63,),
(-130, 117, -36,),
(-130, 145, -20,),
fw4,
    ]
    p5 = fwbbr

    plist = [p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

fwfj = [
(-126, 233, -123,),
(-126, 237, -93,),
(-126, 233, -63,),
(-126, 213, -32,),
(-126, 183, -16,),
fw3,
    ]

def front_wing_front_inner(thick, scl):

    p0 = [
(-96, 220, -123,),
(-96, 225, -93,),
(-96, 220, -67,),
(-96, 205, -42,),
(-94, 183, -29,),
fw0,
    ]
    p1 = [
(-100, 228, -123,),
(-100, 232, -93,),
(-100, 230, -63,),
(-100, 211, -36,),
(-100, 183, -20,),
fw1,
    ]
    p2 = [
(-104, 233, -123,),
(-104, 237, -93,),
(-104, 233, -63,),
(-104, 213, -32,),
(-104, 183, -16,),
fw2,
    ]
    p3 = fwfj

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_wing_front_outer(thick, scl):

    p3 = fwfj
    p4 = [
(-130, 228, -123,),
(-130, 232, -93,),
(-130, 230, -63,),
(-130, 211, -36,),
(-130, 183, -20,),
fw4,
    ]
    p5 = [
(-134, 220, -123,),
(-134, 225, -93,),
(-134, 220, -67,),
(-134, 205, -42,),
(-136, 183, -29,),
fw5,
    ]

    plist = [p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_wing_front_aero(thick, scl):

    p0 = bwbbl
    p1 = [
bwba1,
(-100-5,-20-80, -93,),
(-100-5,-20-90, -63,),
(-100-5,-20-110, -36,),
(-100-5,-20-130, -20,),
bw1,
    ]
    p2 = [
bwba2,
(-104-5,-20-70, -93,),
(-104-5,-20-90, -63,),
(-104-5,-20-110, -32,),
(-104-5,-20-130, -16,),
bw2,
    ]
    p3 = [
bwba3,
(-126-5,-20-60, -93,),
(-126-5,-20-70, -63,),
(-126-5,-20-115, -32,),
(-126-5,-20-130, -16,),
bw3,
    ]
    p4 = [
bwba4,
(-130-5,-20-60, -93,),
(-130-5,-20-70, -63,),
(-130-5,-20-100, -36,),
(-130-5,-20-145, -20,),
bw4,
    ]
    p5 = bwbbr

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_wing_aero_base(thick, scl):

    p0 = bwbl
    p1 = [ bwba1, bwb1,]
    p2 = [ bwba2, bwb2, ]
    p3 = [ bwba3, bwb3, ]
    p4 = [ bwba4, bwb4, ]
    p5 = [ bwab0,bwab0,]

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_wing_back(thick, scl):

    p0 = bwbbl
    p1 = [
bwb1, 
(-100-5,-20-96, -93,),
(-100-5,-20-98, -63,),
(-100-5,-20-117, -36,),
(-100-5,-20-145, -20,),
bw1,
    ]
    p2 = [
bwb2,
(-104-5,-20-91, -93,),
(-104-5,-20-95, -63,),
(-104-5,-20-115, -32,),
(-104-5,-20-145, -16,),
bw2,
    ]
    p3 = [
bwb3,
(-126-5,-20-91, -93,),
(-126-5,-20-95, -63,),
(-126-5,-20-115, -32,),
(-126-5,-20-145, -16,),
bw3,
    ]
    p4 = [
bwb4,
(-130-5,-20-96, -93,),
(-130-5,-20-98, -63,),
(-130-5,-20-117, -36,),
(-130-5,-20-145, -20,),
bw4,
    ]
    p5 = bwbbr

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

bw0 = (-94-5,-20-163, -29,)
bw1 = (-100-5,-20-163, -20,)
bw2 = (-104-5,-20-163, -16,)
bw3 = (-126-5,-20-163, -16,)
bw4 = (-130-5,-20-163, -20,)
bw5 = (-136-5,-20-163, -29,)

bwbb0 = (-96-5,-20-98, -123,)
bwbbl = [
bwbb0,
(-96-5,-20-103, -93,),
(-96-5,-20-108, -67,),
(-96-5,-20-123, -42,),
(-94-5,-20-145, -29,),
bw0,
    ]

bwab0 = (-134-5,-20-108, -123,)

bwbbr = [
bwab0,
(-134-5,-20-103, -93,),
(-134-5,-20-108, -67,),
(-134-5,-20-123, -42,),
(-136-5,-20-145, -29,),
bw5,
    ]

bwba1 = (-100-5,-20-80, -123,)
bwba2 = (-104-5,-20-70, -123,)
bwba3 = (-126-5,-20-45, -123,)
bwba4 = (-130-5,-20-50, -123,)


bwb1 = (-100-5,-20-96, -123,)
bwb2 = (-104-5,-20-95, -123,)
bwb3 = (-126-5,-20-95, -123,)
bwb4 = (-130-5,-20-100, -123,)

bwbl = [ bwbb0, bwbb0]

def rear_wing_floor_strut(thick, scl):

    p0 = [ bwbb0, (-120,-90, -122,)]
    p1 = [ (-85, -110, -123), (-90, -80, -123)]
    p2 = [ (-47, -120, -123), (-50, -90, -123)]

    #plist = [p0, p1, p2, p3, p4, p5]
    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_wing_front(thick, scl):

    p0 = [
(-96-5,-20-220, -123,),
(-96-5,-20-225, -93,),
(-96-5,-20-220, -67,),
(-96-5,-20-205, -42,),
(-94-5,-20-183, -29,),
bw0,
    ]
    p1 = [
(-100-5,-20-228, -123,),
(-100-5,-20-232, -93,),
(-100-5,-20-230, -63,),
(-100-5,-20-211, -36,),
(-100-5,-20-183, -20,),
bw1,
    ]
    p2 = [
(-104-5,-20-233, -123,),
(-104-5,-20-237, -93,),
(-104-5,-20-233, -63,),
(-104-5,-20-213, -32,),
(-104-5,-20-183, -16,),
bw2,
    ]
    p3 = [
(-126-5,-20-233, -123,),
(-126-5,-20-237, -93,),
(-126-5,-20-233, -63,),
(-126-5,-20-213, -32,),
(-126-5,-20-183, -16,),
bw3,
    ]
    p4 = [
(-130-5,-20-228, -123,),
(-130-5,-20-232, -93,),
(-130-5,-20-230, -63,),
(-130-5,-20-211, -36,),
(-130-5,-20-183, -20,),
bw4,
    ]
    p5 = [
(-134-5,-20-220, -123,),
(-134-5,-20-225, -93,),
(-134-5,-20-220, -67,),
(-134-5,-20-205, -42,),
(-136-5,-20-183, -29,),
bw5,
    ]

    plist = [p0, p1, p2, p3, p4, p5]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

sbf1 = (-70,60,-100)
sbf2 = (-60,100,-100)
sbf3 = (-60,150,-100)

fjoin = [sbf1,  (-60, 60, -125), (0, 60, -125)]

def front_floor(thick, scl):

    p0 = fjoin
    p1 = [sbf2,   (-50, 80, -125), (0, 80, -125)]
    p2 = [sbf3,   (-50, 150, -125), (0, 80, -125)]
    p3 = [(-50,200,-100),  (-40, 200, -125), (0, 200, -125)]
    p4 = [rtb5, (0, 230, -100), (0, 210, -125)]

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rfs1 = (-75,-80,-100)
rfs2 = (-70,-100,-100)
rfs3 = (-55,-150,-100)
rfs4 = (-50,-200,-100)

dfr1 = (-90,-20,-100)
dfr2 = (-90,-30,-100)

mfsjoin = [rfs1, (-70, -60, -125), (0, -80, -125)]

rfjoin = [rfs2, (-50, -100, -125), (0, -100, -125)]

mrjoin = [dfr1, (-70, -20, -125), (0, -20, -125)]

def mid_floor(thick, scl):

    p1 = mfsjoin
    p2 = [dfr2, (-70, -30, -125), (0, -30, -125)]
    p3 = mrjoin
    p4 = fjoin

    plist = [p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rfl = [rfs4, (-40, -200, -115), (0, -200, -125)]

def rear_floor(thick, scl):

    p0 = rfl
    p1 = [rfs3, (-40, -150, -125), (0, -150, -125)]
    p2 = rfjoin
    p3 = mfsjoin

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((0, 0, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)


rtb1 = bline[0]
rtb2 = bline[1]
rtb3 = bline[2]
rtb4 = bline[3]
rtb5 = (-20, 230, -100)

rsb1 = rtb1 #(-70, 88, -20)
rsb2 = (-60, 120, -35)
rsb3 = (-55, 150, -50)
rsb4 = (-45, 205, -78)

rsbc1 = [bline[4], (-25, 230, -78), rtb5]

def roof_strut_bonnet_continuation(thick, scl):

    #p0 = [rtb1, (-80, 83, -15), rsb1 ]
    p0 = [rtb1, rtb1, rtb1]
    p1 = [rtb2, (-62, 120, -20), rsb2 ]
    p2 = [rtb3, (-55, 150, -40), rsb3 ]
    p3 = [rtb4, (-40, 208, -68), rsb4 ]
    p4 = rsbc1

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

sbdf1 = (-72, 60,-60)

def side_bonnet(thick, scl):

    p0 = [rsb1, sbdf1, sbf1] 
    p1 = [rsb2, (-60, 100, -70), sbf2]
    p2 = [rsb3, (-53, 150, -80), sbf3]
    p3 = [rsb4, (-50, 200, -90), (-50,200,-100)]
    p4 = [rtb5, rtb5, rtb5]

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

tbw1 = (-70, 105, 0)
tbw2 = (-30, 125, 0)
tbw3 = (0,125,0)

tbc0 = (0, 230, -80)

def top_bonnet(thick, scl):

    p0 = [rtb1, tbw1, tbw2, tbw3] # < windscreen
    p1 = [rtb2, (-60,130,-10), (-30, 150, -10), (0,150,-10)]
    p2 = [rtb3, (-40, 155, -17), (-30, 160, -15), (0,160,-15)]
    p3 = [rtb4, (-25, 212, -50), (-15, 215, -42), (0,215,-40)]
    p4 = [rtb5, rtb5, tbc0, tbc0]

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, thick, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

wse1 = (-83,46,46)
wse2 = (-88,70,20)
wse3 = bline[0] #(-85,88,-5)

wsr1 = (-70,30,60)
wsr2 = (-34, 7, 76)

wsr3 = (0,2,78)

def windscreen(thick, scl):

    #    strut edge
    #       vv
    p0 = [wse1, wsr1, wsr2, wsr3]
    p1 = [wse1, (-68, 48, 47), (-35, 52, 57), (0,52,57)]
    p2 = [wse2, (-65, 80, 30), (-32, 88, 30), (0,88,30)]
    p3 = [wse3, tbw1, tbw2, tbw3]
    p4 = [wse3, tbj3, tbj2, tbj1]

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, -thick, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rfsd0 = (-82,10,55.5)
rfsd1 = (-92,42,35)
rfsd2 = (-92,70,0)

rsbj = [wse3, (-92, 76, -10), (-93,72,-10)]

def roof_strut_bonnet_joiner(thick, scl):

    p0 = rsbj
    p1 = [wse3, tb12, tb13]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, thick, thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def roof_front_strut(thick, scl):

    #    windscreen
    #       vv
    p0 = [wse1, (-82, 30, 50), rfsd0]
    p1 = [wse1, (-89, 43, 40), rfsd1]
    p3 = [wse2, (-90, 70, 10), rfsd2]
    p4 = rsbj

    plist = [p0, p1, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rrs1 = (-58, -120, 76)
rrs2 = (-50, -95, 79)
rrs21 = (-70, -110, 75)

rst0 = (-80,-50,68)
rst1 = (-80,-80,68)
rst2 = (-80,-82,68)
rst3 = (-80,-85,68)

rrs22=(-80,-95,68)


rjl = [rrs22, (-60,-80,80), (-20,-70,80), (0, -68, 80)]

def roof_front(po, scl):

    p1 = rjl
    p2 = [rrs22, (-60,-75,80), (-20,-65,80), (0, -65, 80)]
    p3 = [rst2, (-60,-70,80), (-20,-60,80), (0, -60, 80)]
    p4 = [rst0, (-65,-40,80), (-20,-40,80), (0, -40, 80)]
    p5 = [rfsd0, (-68,0,71), (-20,-10,78), (0, -10, 80)]
    p6 = [(-83,46,46), wsr1, wsr2, wsr3]

    plist = [p1, p2, p3, p4, p5, p6]

    if spline_me: plist = spline_double_divide(plist)

    po = Offset((0,0,po))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rrs3 = (-30,-85,80)
rrs4 = (0,-80, 80)

rsl = [rrs1, rrs2, rrs3, rrs4]

def roof_rear(po, scl):

    p0 = rsl
    p1 = [rrs21, (-60,-95,80), (-30,-80,80), (0,-75, 80)]
    p2 = [rrs22, (-60,-85,80), (-25,-75,80), (0, -70, 80)]
    p3 = rjl

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist)

    po = Offset((0,0,po))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rws1 = (-55, -170, 61)
rws2 = (-52, -210, 38)
rws3 = (-48, -230, 22)

rwsl = [rws3,(-15,-230,24),(-10,-230,24),(0,-230,24)]

def rear_windscreen(po, scl):

    p0 = rsl
    p1 = [rws1,(-40,-150,72),(-20,-140,75),(0,-140,75)]
    p2 = [rws2,(-30,-200,48),(-20,-180,60),(0,-180,60)]
    p3 = [rws3,(-15,-230,24),(-10,-230,24),(0,-230,24)]

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0,0,-po))
    return SurfaceFromCurves(plist).create_surface(po, scl)


rtp0 = (-18, -245, 10)
rtp1 = (-10, -252, 10)
rtp2 = (-5, -255, 10)
rtp3 = (0, -255, 10)

rms0 = rtp0
rms1 = (-20, -245, -50)
rms2 = (-20,-255,-100)

ref0 = (-25, -240, -100)

def rear_end_floor(po, scl):

    p0 = rfl
    p1 = [ref0, (-20, -230, -105), (0, -220, -125)]
    p2 = [rms2, (-20, -255, -100), (0, -255, -100)]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((po,0,0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_mid_strut(po, scl):

    p0 = [sfpr1, sfpr11, sfpm1, rfs4]
    p1 = [rms0, rms0, (-25, -230, -50), (-25, -240, -100)]
    p2 = [rms0, rms0, rms1, rms2]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((po,0,0))
    return SurfaceFromCurves(plist).create_surface(po, scl)


def rear_top_left(po, scl):

    p0 = rwsl
    p1 = [rtp0, rtp1, rtp2, rtp3]

    plist = [p0, p1]#, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0,0,-po))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def rear_top_mid(po, scl):

    p0 = rrs 
    p1 = [rtp0, rtp0, rtp0]

    plist = [p0, p1]#, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0,0,-po))
    return SurfaceFromCurves(plist).create_surface(po, scl)


def back_plate(po, scl):

    p0 = [rtp0, rtp1, rtp2, rtp3]
    p1 = [rms1, (-10, -247, -50), (-5, -250, -50), (-0, -250, -50)]
    p2 = [rms2, (-10,-255,-100), (-5,-255,-100), (-0,-255,-100)]

    plist = [p0, p1,p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0,0,-po))
    return SurfaceFromCurves(plist).create_surface(po, scl)


srs1 = (-110, -110, 5)
srs2 = (-108, -115, 6)
srs3 = (-106, -120, 8)

msrs1 = (-90,-90,60)
msrs2 = (-104,-100,35)

sws1 = (-88, -100, 60)
sws2 = (-102, -110, 35)

def side_roof_strut(thick, scl):

    p0 = [rst2, rst3, rrs22]
    p1 = [msrs1, (-89, -95, 60), sws1]
    p2 = [msrs2, (-103, -105, 35), sws2]
    p3 = [srs1, srs2, srs3]

    #plist = [p1, p2, p3, p4, p5]
    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

sfpr1 = (-60, -230, 10)
sfpr2 = (-90, -150, 10)
sfpdf1 = (-75, -100, -50)
sfpm1 = (-50, -200, -50)
sfpm2 = (-50, -200, -54)

srs11 = (-110, -110, 0)
sfpr11 = (-50, -200, 5)

def side_floor_panel(thick, scl):

    p0 = [srs1, srs2, srs3, sfpr2, sfpr1]
    p1 = [srs1, (-108, -115, 1), (-106, -120, 3), (-90, -150, 5), sfpr11]
    p2 = [sfpdf1, (-72, -110, -50),(-68, -120, -50), (-65, -150, -50), sfpm1]
    p3 = [rfs1, rfs2, rfs2, rfs3, rfs4]

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

dfl = [rsb1, (-115, 0, -10), (-125, -40, -8), (-122, -70, -2), srs1]

def door_frame(thick, scl):

    p0 = [rfsd0, rst0, rst0, rst1, rst2]
    p1 = [rfsd1, (-92, 0, 50), (-95, -20, 50), (-98, -50, 50), msrs1]
    p2 = [rfsd2, (-105, 10, 30), (-110, -20, 30), (-110, -50, 30), msrs2]
    p3 = dfl

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

odp = [(-125, 65,-50), (-128, 50,-50), (-136, -20, -50), (-136, -60, -48), (-126, -100, -47)]

def outer_door_panel(thick, scl):

    p0 = dfl
    p1 = [(-122, 72, -35), (-125, 30, -30), (-136, -30, -20), (-136, -60, -18), (-130, -110, -12)]
    p2 = odp

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

odp2 = [(-115,55,-90), (-117,40,-90), (-125,-20,-87), (-120, -60, -87), (-115,-95,-87)]

def outer_door_panel2(thick, scl):

    p0 = odp
    p1 = [(-110, 65,-50), (-112, 50,-50), (-125, -20, -53), (-120, -60, -53), (-115, -100, -53)]
    p3 = odp2

    plist = [p0, p1, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def outer_door_panel3(thick, scl):

    p0 = odp2
    p1 = [(-130,55,-90), (-130,40,-90), (-136,-20,-90), (-136, -60, -90), (-130,-95,-90)]
    p3 = [(-130,55,-100), (-130,40,-100), (-136,-20,-100), (-136, -60, -100), (-130,-95,-100)]

    plist = [p0, p1, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def lower_door_frame(thick, scl):

    p0 = dfl
    p1 = [sbdf1, (-80, -40, -50), (-80, -50, -50), (-80, -60, -50), sfpdf1]
    p2 = [sbf1, dfr1, dfr1, dfr2, rfs1]

    plist = [p0, p1, p2]

    if spline_me: plist = spline_double_divide(plist, step_v=0.1, step_h=.1)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

rrs = [sfpr1, (-56, -230, 18), rws3]

def roof_rear_strut(thick, scl):

    p0 = [rrs22, rrs21, rrs1 ]
    p1 = [(-75, -140, 58), (-67, -154, 58),rws1,]
    p2 = [(-65, -190, 35), (-60, -200, 38),rws2,]
    p3 = rrs
    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def side_window(thick, scl):

    p0 = [rrs22, rrs22, rrs22]
    p1 = [(-75, -140, 58), (-85, -120, 58), sws1]
    p2 = [(-50, -190, 35), (-90, -140, 35), sws2]
    p3 = [sfpr1, sfpr2, srs3]

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

fasl1 = (-118, 240, -32)
fasl2 = (-118, 250, -30)
fasl3 = (-118, 200, -20)

fasr1 = (-5, 240, -32)
fasr2 = (-5, 250, -30)
fasr3 = (-30, 160, -15)

facb0 = [ fasl1, (-95, 230, -30), (-85, 230, -30), (-70, 230, -30),
            fasr1 ]
facb1 = [ fasl3 , (-105, 190, -10), (-95, 175, -8), (-75, 165, -8),
            fasr3 ]
facb2 = [ (-118, 250, -30), (-103, 235, -10), (-85, 230, -8), (-70, 230, -10),
            (-10, 250, -30)]

facf3 = [ (-118, 250, -30), (-99, 248, -18), (-85, 248, -18), (-70, 248, -18),
            (-10, 250, -30)]

facf4 = [ (-118, 250, -30), (-99, 240, -12), (-85, 240, -12), (-70, 240, -14),
            (-10, 250, -30)]

facf5 = [ fasl2, (-95, 254, -24), (-85, 254, -24), (-60, 254, -24),
            fasr2 ]

fatp0 = (0, 250, -30)
fatp1 = (0, 200, -30)
fatp2 = tbc0 #(0, 230, -80)

def front_aero_wing_panel(thick, scl):

    p0 = [fasl2, (-118, 230, -80)]
    p1 = [fasl1, (-118, 223, -60)]
    p2 = [fasl3, (-118, 210, -40)]
    p3 = [fasl3, (-118, 200, -34)]

    plist = [p0, p1, p2, p3]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_front_panel(thick, scl):

    p0 = [fasr2, (0, 250, -30)]
    p1 = [fasr2, fatp2]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_side_panel(thick, scl):

    p0 = [fasr2, fatp2 ]
    p1 = [fasr2, (-14, 210, -50)]
    p2 = [fasr2, (-24, 180, -40)]
    p3 = [fasr2, (-26, 175, -30)]
    p4 = [fasr2, fasr3]

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_top_panel(thick, scl):

    p0 = [fatp0, fasr2]
    p1 = [fatp1, fasr3]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_side1(thick, scl):

    p0 = [fasl1, fasl2]
    p1 = [fasl3, fasl3]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_side2(thick, scl):

    p0 = [fasr1, fasr2]
    p1 = [fasr3, fasr3]

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, 0, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_front(thick, scl, colour):

    p0 = facb0
    p1 = facf5
    p2 = facf3
    p3 = facf4
    p4 = facb2

    plist = [p0, p1, p2, p3, p4]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((thick, thick, 0))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_bottom(thick, scl):

    p0 = facb0
    p1 = facb1

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def front_aero_top(thick, scl):

    p0 = facb1
    p1 = facb2

    plist = [p0, p1]

    if spline_me: plist = spline_double_divide(plist, step_v=0.15, step_h=.15)

    po = Offset((0, 0, -thick))
    return SurfaceFromCurves(plist).create_surface(po, scl)

def create_car(t, _scl, mirror, colour):

    scl = [-_scl*mirror, _scl, _scl]

    strut_thickness = 0.1
    roof_thickness = 0.1
    windscreen_thickness = 0.1
    bonnet_thickness = 0.1
    floor_thickness = 0.1
    door_thickness = 0.1
    wing_thickness = 0.1
    grille_thickness = 1.0
    arch_thickness = 1.0
    side_aero_thickness = 1.0
    body_thickness = 0.1

    #c = color([0.6, 1.0, 0.6]) 
    if colour:
        c = color(colour)
    else:
        c = color([1.0, 0.6, 0.2])
    c.add(t)
    t.add(roof_rear(roof_thickness, scl))
    t.add(roof_front_strut(strut_thickness, scl))
    t.add(roof_rear_strut(strut_thickness, scl))
    t.add(side_roof_strut(strut_thickness, scl))
    t.add(roof_strut_bonnet_continuation(bonnet_thickness, scl))
    t.add(front_floor(floor_thickness, scl))
    t.add(rear_floor(floor_thickness, scl))
    t.add(lower_door_frame(door_thickness, scl))
    #t.add(rear_wing_front(wing_thickness, scl))
    #t.add(rear_wing_back(wing_thickness, scl))
    #t.add(rear_wing_front_aero(wing_thickness, scl))
    #t.add(front_wing_front_outer(wing_thickness, scl))
    t.add(test_rear_wing_rear_inner(wing_thickness, scl))
    t.add(test_front_wing_front_inner(wing_thickness, scl))
    t.add(test_rear_wing_front_inner(wing_thickness, scl))
    #t.add(front_wing_back_inner(wing_thickness, scl))
    #t.add(front_wing_back_outer(wing_thickness, scl))
    #t.add(front_wing_back_aero_outer(wing_thickness, scl))
    #t.add(front_wing_back_aero_inner(wing_thickness, scl))
    #t.add(rear_wing_aero_base(wing_thickness, scl))
    #t.add(front_wing_aero_base(wing_thickness, scl))
    #t.add(front_aero_bottom(wing_thickness, scl))
    #t.add(front_aero_top(wing_thickness, scl))
    #t.add(front_aero_side1(wing_thickness, scl))
    #t.add(front_aero_side2(wing_thickness, scl))
    t.add(rear_arch_lid(arch_thickness, scl))
    t.add(rear_arch_body_joiner(arch_thickness, scl))
    t.add(rear_spoiler(arch_thickness, scl))
    t.add(rear_mid_strut(body_thickness, scl))
    t.add(outer_door_panel2(body_thickness, scl))
    
    w = union()
    #c = color([0.2, 0.2, 0.2, 0.8])
    c = color([0.2, 0.2, 0.2, 0.5])
    c.add(w)
    w.add(windscreen(windscreen_thickness, scl))
    w.add(side_window(windscreen_thickness, scl))
    w.add(door_frame(windscreen_thickness, scl))
    w.add(rear_windscreen(windscreen_thickness, scl))
    w.add(headlight_panel(windscreen_thickness, scl))

    t.add(c)

    b = union()
    if colour:
        c = color(colour)
    else:
        #c = color([0.2, 0.2, 1.0, 1.0]) 
        c = color([1.0, 0.2, 1.0, 1.0])
    c.add(b)
    #b.add(top_bonnet(bonnet_thickness, scl))
    b.add(side_bonnet(bonnet_thickness, scl))
    b.add(side_floor_panel(body_thickness, scl))
    b.add(roof_front(roof_thickness, scl))
    b.add(mid_floor(floor_thickness, scl))
    b.add(rear_wing_floor_strut(wing_thickness, scl))
    b.add(front_wing_floor_strut(wing_thickness, scl))
    #t.add(front_aero_top_panel(wing_thickness, scl))
    #t.add(front_aero_side_panel(wing_thickness, scl))
    #b.add(front_aero_front_panel(wing_thickness, scl))
    #b.add(front_aero_wing_panel(-side_aero_thickness, scl))
    #b.add(test_panel(wing_thickness, scl))
    #b.add(test_rear_panel(wing_thickness, scl))
    b.add(rear_top_mid(body_thickness, scl))
    b.add(rear_top_left(body_thickness, scl))
    b.add(rear_end_floor(body_thickness, scl))
    t.add(back_plate(body_thickness, scl))
    b.add(outer_door_panel(body_thickness, scl))
    b.add(outer_door_panel3(body_thickness, scl))

    t.add(c)

    b = union()
    if colour:
        c = color(colour)
    else:
        c = color([1.0, 0.0, 0.0, 1.0]) 
    c.add(b)
    b.add(rear_arch_side(arch_thickness, scl))
    b.add(rear_arch_inside(arch_thickness, scl))
    b.add(rear_arch_front(arch_thickness, scl))
    t.add(c)

    b = union()
    if colour:
        c = color(colour)
    else:
        c = color([1.0, 1.0, 1.0, 1.0]) 
    c.add(b)
    #b.add(front_aero_front(wing_thickness, scl))
    t.add(c)

    b = union()
    c = color([0.5, 0.5, 0.5, 1.0]) 
    c.add(b)
    #front_lower_grille(b, grille_thickness, scl)
    #front_grille(b, grille_thickness, scl)
    #rear_grille(b, grille_thickness, scl)
    b.add(rear_wing_under_scoop(grille_thickness, scl))
    b.add(rear_wing_over_scoop(grille_thickness, scl))
    b.add(front_wing_under_scoop6(grille_thickness, scl))
    #t.add(front_wing_under_scoop5(grille_thickness, scl))
    #t.add(front_wing_under_scoop4(grille_thickness, scl))
    #t.add(front_wing_under_scoop3(grille_thickness, scl))
    b.add(front_wing_under_scoop2(grille_thickness, scl))
    b.add(front_wing_under_scoop(grille_thickness, scl))
    b.add(front_wing_over_scoop(grille_thickness, scl))
    #rear_lower_grille(b, grille_thickness, scl)
    #rear_lower_grille(b, grille_thickness, scl)
    t.add(c)

    b = union()
    c = color([0.8, 0.8, 0.8, 1.0])
    c.add(b)
    b.add(hubcap_front_left(_scl, mirror))
    b.add(hubcap_back_left(_scl, mirror))
    t.add(c)

    b = union()
    c = color([0.2, 0.2, 0.2, 1.0])
    c.add(b)
    #b.add(bullbars_front(_scl, mirror))
    b.add(bullbars_rear(_scl, mirror))
    b.add(wheel_front_left(_scl, mirror))
    b.add(wheel_back_left(_scl, mirror))
    t.add(c)

    b = union()
    if colour:
        c = color(colour)
    else:
        c = color([1.0, 0.2, 0.2, 1.0]) 
    b.add(test_bonnet1(wing_thickness, scl))
    b.add(front_side_bonnet_panel(wing_thickness, scl))
    b.add(test_side_bonnet_top_panel(wing_thickness, scl))
    b.add(test_side_bonnet_behind_wheel(wing_thickness, scl))
    b.add(nose(bonnet_thickness, scl))

    c.add(b)

    t.add(c)

    b = union()
    if colour:
        c = color(colour)
    else:
        c = color([0.2, 0.2, 1.0, 1.0]) 
    c.add(b)
    t.add(c)


    b = union()
    if colour:
        c = color(colour)
    else:
        c = color([0.2, 1.0, 0.2, 1.0]) 
    b.add(test_bonnet2(wing_thickness, scl))
    b.add(test_bonnet3(wing_thickness, scl))
    c.add(b)

    t.add(c)

def create_test(t, _scl, mirror):

    scl = [-_scl*mirror, _scl, _scl]

    strut_thickness = 0.1
    roof_thickness = 0.1
    windscreen_thickness = 0.1
    bonnet_thickness = 0.1
    floor_thickness = 0.1
    door_thickness = 0.1
    wing_thickness = 0.1
    grille_thickness = 1.0
    arch_thickness = 1.0
    side_aero_thickness = 1.0
    body_thickness = 0.1

    b = union()
    c = color([1.0, 0.2, 0.2, 1.0])
    b.add(test_bonnet1(wing_thickness, scl))
    c.add(b)

    t.add(c)


    b = union()
    c = color([0.2, 1.0, 0.2, 1.0])
    b.add(test_bonnet2(wing_thickness, scl))
    b.add(test_bonnet3(wing_thickness, scl))
    c.add(b)

    t.add(c)

    b = union()
    #c = color([0.2, 0.2, 1.0, 1.0])
    c = color([0.2, 0.2, 0.2, 0.3])
    #b.add(test_panel(wing_thickness, scl))
    c.add(b)

    t.add(c)

    #c = color([0.6, 1.0, 0.6])
    c = color([1.0, 0.6, 0.2])
    c.add(t)
    t.add(roof_strut_bonnet_continuation(bonnet_thickness, scl))
    t.add(roof_strut_bonnet_joiner(strut_thickness, scl))
    t.add(roof_front_strut(strut_thickness, scl))
    t.add(side_bonnet(bonnet_thickness, scl))
    t.add(rear_arch_front(arch_thickness, scl))
    t.add(rear_arch_inside(arch_thickness, scl))
    t.add(rear_arch_side(arch_thickness, scl))

    b = union()
    c = color([0.2, 0.2, 0.2, 1.0])
    c.add(b)
    b.add(wheel_front_left(_scl, mirror))
    b.add(wheel_back_left(_scl, mirror))
    t.add(c)

    b = union()
    c = color([0.2, 0.2, 1.0, 1.0])
    c.add(b)
    b.add(nose(bonnet_thickness, scl))
    t.add(c)

    w = union()
    c = color([0.2, 0.2, 0.2, 0.5])
    c.add(w)
    w.add(windscreen(windscreen_thickness, scl))
    w.add(headlight_panel(windscreen_thickness, scl))
    t.add(c)

if __name__ == '__main__':

    scl = 0.1
    t = union()
    c = [0.6, 0.5, 0.8, 1.0]
    c = None
    create_car(t, scl, -1.0, c)
    create_car(t, scl, 1.0, c)
    #create_test(t, scl, -1.0)
    #create_test(t, scl, 1.0)

    scad_render_to_file(t, "car_prog.scad")
    f = open("car_model.scad", "w")
    f.write("""include<car_prog.scad>;
//%import_stl(filename = "car_model.stl");
    """)
    f.close()

